/* 
	R only reads files up to stata 12
	We use stata13 so save files for use in R using saveold rather than save
*/

cd "C:\Users\lieke\Desktop\Proefschrift\Gridded sampling\Data\DHS data"
 
use "Original data\NM_2013_DHS_06072017_stata\nmhr61dt\NMHR61FL.DTA", clear

keep hhid hv001 hv002 hv005 hv024 hv025 ///  hv010 nr eligable women construct in mom file hv219 hv220 (hh head sex and age)
	 hv201 hv205 hv213 hv216 hv225 hv226 hv270 /// hv214 hv215 wall and roof
	 hv101_?? hv104_?? hv105_?? hv109_??
	 
forvalues i = 1/9 {
	foreach var in hv101 hv104 hv105 hv109 {
		rename `var'_0`i' `var'_`i'
	}
}

reshape long hv101_ hv104_ hv105_ hv109_, i(hhid) j(pid) 
drop if hv101_ >= .   

//check
gen temp = 1 if pid[_n+1] != pid[_n] + 1 & pid[_n+1] != 1 
//only generated for last observation, so hh members were filled 1,2,3 not leaving gaps. 
drop temp
egen hhsize = max(pid), by(hhid)
label var hhsize "number of household members" 

rename hv001 cluster_nr
rename hv002 hh_nr
rename hv005 hh_weight
rename hv024 region
rename hv025 urban_rural
label define urban_rural 1 "Urban" 2 "Rural"
label values urban_rural urban_rural

gen relationship = 1 if hv101_ == 1
replace relationship = 2 if hv101_== 2 
replace relationship = 3 if hv101_ == 3 | hv101_ == 4 | hv101_ == 11
replace relationship = 4 if hv101_ == 5
replace relationship = 5 if hv101_ >= 6 & hv101_ <= 10
replace relationship = 6 if hv101_ == 12 | hv101_ == 98
label define relationship 0 "0" ///
						  1 "head" ///
						  2 "spouse" ///
						  3 "child" ///
						  4 "gchild" ///
						  5 "extend" ///
						  6 "other" 
label values relationship relationship
drop hv101_
						  
rename hv104_ sex
label define sex 1 "Male" 2 "Female"
label values sex sex

gen temp = sex if relationship == 1
sort hhid
by hhid: egen sex_hhhead = max(temp)
label values sex_hhhead sex
drop temp

gen age = 0 if hv105_ == 0 
replace age = 1 if hv105_ >= 1 & hv105_ < 5
replace age = 2 if hv105_ >= 5 & hv105_ < 10
replace age = 3 if hv105_ >= 10 & hv105_ < 15
replace age = 4 if hv105_ >= 15 & hv105_ < 20
replace age = 5 if hv105_ >= 20 & hv105_ < 25
replace age = 6 if hv105_ >= 25 & hv105_ < 30
replace age = 7 if hv105_ >= 30 & hv105_ < 35
replace age = 8 if hv105_ >= 35 & hv105_ < 40
replace age = 9 if hv105_ >= 40 & hv105_ < 45
replace age = 10 if hv105_ >= 45 & hv105_ < 50
replace age = 11 if hv105_ >= 50 & hv105_ < 55
replace age = 12 if hv105_ >= 55 & hv105_ < 60
replace age = 13 if hv105_ >= 60 & hv105_ < 65
replace age = 14 if hv105_ >= 65 & hv105_ < 75
replace age = 15 if hv105_ >= 75 & hv105_ < 98 // 98: don't know, set to missing
label define age 0 "0" ///
				 1 "1 - 4" ///
				 2 "5 - 9" ///
				 3 "10 - 14" ///
				 4 "15 - 19" ///
				 5 "20 - 24" ///
				 6 "25 - 29" ///
				 7 "30 - 34" ///
				 8 "35 - 39" ///
				 9 "40 - 44" ///
				 10 "45 - 49" ///
				 11 "50 - 54" ///
				 12 "55 - 59" ///
				 13 "60 - 64" ///
				 14 "65 - 74" ///
				 15 "75+" 
label values age age
drop hv105_ 

gen temp = age if relationship == 1
sort hhid
by hhid: egen age_hhhead = max(temp)
label values age_hhhead age
drop temp
			 
gen education = 1 if hv109_ == 0 
replace education = 2 if hv109_ == 1 
replace education = 3 if hv109_ == 2 | hv109_ == 3
replace education = 4 if hv109_ == 4 
replace education = 5 if hv109_ == 5 
replace education = 7 if hv109_ == 8
label define education 1 "No formal" ///
					   2 "Incomplete primary" ///
					   3 "Complete primary" ///
					   4 "Complete secondary" ///
					   5 "Complete tertiary" ///
					   6 "Others" ///
					   7 "Don't know" 
label values education education
drop hv109_

gen temp = education if relationship == 1
sort hhid
by hhid: egen educ_hhhead = max(temp)
label values educ_hhhead education
drop temp	
	
gen water = 0 if hv201 == 11 | hv201 == 12 | hv201 == 13 | /// piped into dwelling, pied into yard/plot, public tap/standpipe
				 hv201 == 21 | hv201 == 31 | hv201 == 41 | /// tube well or borehole, protected well, protected spring
				 hv201 == 51 | hv201 == 71  			   // rainwater, bottled water

replace water = 1 if hv201 == 32 | hv201 == 42 | 		   /// unprotected well, unprotected spring
					 hv201 == 61 | hv201 == 62 | 		   /// tanker truck, cart with small tank
					 hv201 == 43						   //  river/dam/lake/ponds/stream/canal/irrig

label define water 0 "improved water source" 1 "unimproved water source"
label values water water
// other (96) is now labeled as missing. 
// 2.23% of households, 0.11% was already missing
drop hv201

gen toilet = 0 if hv205 == 11 | hv205 == 12 | hv205 == 13 | /// flush to piped sewer system, to septic tank, to pit latrine
				  hv205 == 21 | hv205 == 22 | 				/// ventilated improved pit latrine, pit latrine with slab
				  hv205 == 41 								//  composting toilet

replace toilet = 1 if hv205 == 14 |	hv205 == 15 |			///	 flush to somewhere else, flush, don't know where
					  hv205 == 42 | hv205 == 43 |			///  bucket toilet, hanging toilet/latrine
					  hv205 == 23 | hv205 == 31	|			///  pit latrin without slab/open pit, no facility/bush/field
					  hv225 == 1							//   toilet shared with others. 

replace toilet = . if hv225 == . & toilet == 0 				//
label define toilet 0 "improved toilet facility" 1 "unimproved toilet facility"
label values toilet toilet
// other (96) is now labeled as missing and those with improved toilets but missing data on sharing. 
// resp 0.26% of households and 0.04% of households, 0.04% was already missing					  
drop hv205 hv225

gen structure = 0 if hv213 == 31 | hv213 == 32 | 			 		/// parquet/polished floor, vinyl/asphalt strips
					 hv213 == 33 | hv213 == 34 | hv213 == 35		//  ceramic tiles, cement, carpet
replace structure = 1 if hv213 == 11 | hv213 == 12 | hv213 == 13| 	/// earth/sand, dung, mud/clay
						 hv213 == 21 | hv213 == 22					//	wood planks, palm/bamboo
label define structure 0 "durable structure" 1 "non-durable structure"
label values structure structure	
						   
// other(96) is set to missing
// 0.46% of households, 0.10% was already missing. 
drop hv213

gen ind_room = hhsize/hv216
gen space = 0 if ind_room <= 3  
replace space = 1 if ind_room > 3 & ind_room < . 
label define space 0 "adequate space (<=3 per room)" 1 "inadequate space (> 3 per room)" 
label values space space
drop hv216 ind_room

gen slum = (water == 1 | toilet == 1 | structure == 1 | space == 1) ///
	if water < . & toilet < . & structure < . & space < .  

label define slum 0 "non-slum household" 1 "slum household"
label values slum slum

sum water toilet structure space slum
	 
/*
improved water: 87.3%
improved toilet: 44.78%
adequate structure: 26.05%
adequate space: 77.6%
improved water and toilet: 44.6% --> i.e. if improved water usually also improved toilet
  and adequate structure: 4.02% --> here we might be overdoing it with the definition..
     and adequate space: 4.7%
*/ 

gen fuel = 0 if hv226 >= 1 & hv226 <= 6 // electricity, lpg, natural gas, biogas, kerosene, paraffin, coal/lignite
replace fuel = 1 if hv226 >= 7 & hv226 <= 11 // charcoal, wood, agricultural crop, animal dung
label define fuel 0 "non-solid fuel" 1 "solid fuel"
label values fuel fuel	

// other(96) and no food cooked in household (95) are set to missing
// resp 0.03% and 0.08% of households, 0.01% was already missing. 
drop hv226

rename hv270 wealth_index
	 
merge 1:1 cluster_nr hh_nr pid using "Generated data\DHS_mom2_recode.dta", keep (1 3)
drop _merge

merge 1:1 cluster_nr hh_nr pid using "Generated data\DHS_child_recode.dta", keep (1 3)
drop _merge

saveold "Generated data\DHS_ind_recode.dta", version(12) replace
	
