####
## Step 0.1 Prepping modis, health, and school files so that they can be processed similar to world pop files. 
####

##### 0. initialization
library(gdalUtils)
library(raster)
library(rgdal)
library(plyr)

setwd("C:/Users/lieke/Desktop/Proefschrift/Gridded sampling/HDF_NPP_Convert_HDF_Tif")

##### 1. NNP
hdf.files_source <- list.files(pattern=".hdf")

hdf.files <- strsplit(x = hdf.files_source, split = "[.]")                       # split by point "[.]"
hdf.files <- ldply(hdf.files)                                                    # convert resulting list to dataframe
hdf.files1 <- paste(hdf.files$V1,"_", hdf.files$V2, "_", hdf.files$V3, "_ET", sep = "")             # paste back only the desired pieces
hdf.filesnd <- as.vector(hdf.files1) 

gdalinfo("MOD17A3.A2012001.h19v10.055.2016008104524.hdf")
sds <- get_subdatasets("MOD17A3.A2012001.h19v10.055.2016008104524.hdf") # what is in HDF
filename1 <- paste(hdf.filesnd, ".tif", sep = "")  # Tif output name

# loop to convert the first layer which is NPP in HDF file to tif
for (i in 1:length(hdf.files_source)) {
  sds <- get_subdatasets(hdf.files_source[i]) 
  gdal_translate(sds[2], dst_dataset = filename1[i])
}

# Now resample to 100 m grid using nearest neighbor method
setwd("C:/Users/lieke/Desktop/Proefschrift/Gridded sampling/Data/Spatial Covariates")

NPP_1 <- raster("HDF_NPP_Convert_HDF_Tif/MOD17A3_A2012001_h19v10_ET.tif")
NPP_2 <- raster("HDF_NPP_Convert_HDF_Tif/MOD17A3_A2012001_h19v11_ET.tif")
NPP_3 <- raster("HDF_NPP_Convert_HDF_Tif/MOD17A3_A2012001_h20v10_ET.tif")
NPP_1km <- merge(NPP_1, NPP_2, NPP_3)

rm(NPP_1, NPP_2, NPP_3)
gc()

rm(dataset_name, filename1, gdalPath, hdf.files_source, hdf.files1, hdf.filesnd, i, modis_archive_path, modis_out_path, out_SDSstring, sds)

admin_grid <- raster("WorldPop cov/nam_grid_100m_ccidadminl0.tif")
NPP_1km <- projectRaster(NPP_1km, crs = proj4string(admin_grid))
NPP_1km <- resample(NPP_1km, admin_grid, method = "ngb")
NPP_1km <- crop(NPP_1km, admin_grid)
NPP_1km <- mask(NPP_1km, admin_grid)
rm(admin_grid, hdf.files)
gc()

#writeRaster(NPP_1km, "Data/Spatial Covariates/MODIS cov/nam_grid_100m_NPP_2012.tif", "GTiff")
#gives memory issues, so do it in parts
tr <- blockSize(NPP_1km)
s <- writeStart(NPP_1km, "Generated/nam_grid_100m_NPP_2012.tif", "GTiff", overwrite = TRUE)
for (i in tr$n:1) {
  v <- getValuesBlock(NPP_1km, row=tr$row[i], nrows=tr$nrows[i])
  s <- writeValues(s, v, tr$row[i])
}
s <- writeStop(s)

rm(list = ls())

##### 2. Schooling files

setwd("C:/Users/lieke/Desktop/Proefschrift/Gridded sampling/Data/Spatial Covariates")

admin_grid <- raster("WorldPop cov/nam_grid_100m_ccidadminl0.tif")
schools <- raster("School and health/edu_ocha_v2/w001001.adf")
#resolution is off by a small bit, fix by resampling
schools <- resample(schools, admin_grid, method = "ngb")
schools <- crop(schools, admin_grid)
schools <- mask(schools, admin_grid)

#writeRaster(schools, "Generated/nam_grid_100m_schools_dst.tif", "GTiff")
#gives memory issues, so do it in parts
tr <- blockSize(schools)
s <- writeStart(schools, "Generated/nam_grid_100m_schools_dst.tif", "GTiff", overwrite = TRUE)
for (i in tr$n:1) {
  v <- getValuesBlock(schools, row=tr$row[i], nrows=tr$nrows[i])
  s <- writeValues(s, v, tr$row[i])
}
s <- writeStop(s)

rm(schools)

##### 3. health files
hfacilities <- raster("School and health/hf_ocha_v3/w001001.adf")
#resolution is off by a small bit, fix by resampling
hfacilities <- resample(hfacilities, admin_grid, method = "ngb") 
hfacilities <- crop(hfacilities, admin_grid)
hfacilities <- mask(hfacilities, admin_grid)

#writeRaster(hfacililies, "Generated/nam_grid_100m_hfacilities_dst.tif", "GTiff")
#gives memory issues, so do it in parts
tr <- blockSize(hfacilities)
s <- writeStart(hfacilities, "Generated/nam_grid_100m_hfacilities_dst.tif", "GTiff", overwrite = TRUE)
for (i in tr$n:1) {
  v <- getValuesBlock(hfacilities, row=tr$row[i], nrows=tr$nrows[i])
  s <- writeValues(s, v, tr$row[i])
}
s <- writeStop(s)
