####
## Phase B - step 5. Generate synthetic population from Census data
####

##### 0. initialize

library(foreign)
library(plyr)
library(simPop)
library(rgdal)

setwd("C:/Users/lieke/Desktop/Proefschrift/Gridded sampling")

set.seed(1)  #For population 1 we use set.seed(1), for population 2 we use set.seed(2) etc. 

getmode <- function(v) {
  uniqv <- unique(v)
  if (sum(!is.na(uniqv)) > 0) {
    uniqv <- uniqv[!is.na(uniqv)]
  }
  uniqv[which.max(tabulate(match(v, uniqv)))]
}

##### 1. load and merge data

## housing file
census_housing <- read.dta("Data/Census PUMS/Original data/HOUSING RECORDS.dta")
names(census_housing) <- tolower(names(census_housing))
keep_housing <- c("region", "constituency", "urban_rural", "hh_type", "household_serial_number", 
                  "h4", "h7", "h8a", "h9", "h10", "hwgt")
census_housing <- census_housing[, (names(census_housing) %in% keep_housing)]
names(census_housing) <- c("region", "constituency", "urban_rural", "hh_type", "household_serial_number", 
                           "sleeping_rooms", "structure", "fuel", "water", "toilet", "hwgt")
summary(census_housing)

#recategorize corresponding to DHS
#structure
census_housing$structure_copy <- census_housing$structure
census_housing$structure <- as.numeric(census_housing$structure)
census_housing$structure[census_housing$structure %in% c(2,5,6)] <- 0
census_housing$structure[census_housing$structure %in% c(1,3,4)] <- 1
census_housing$structure[census_housing$structure %in% c(7)] <- NA
census_housing$structure <- as.factor(census_housing$structure)
levels(census_housing$structure) <- c("Durable floor","Non-durable floor")
#check
table(census_housing$structure_copy, census_housing$structure)

#fuel
census_housing$fuel_copy <- census_housing$fuel
census_housing$fuel <- as.numeric(census_housing$fuel)
census_housing$fuel[census_housing$fuel %in% c(1:4,8)] <- 0
census_housing$fuel[census_housing$fuel %in% c(5:7)] <- 1
census_housing$fuel[census_housing$fuel %in% c(9)] <- NA
census_housing$fuel <- as.factor(census_housing$fuel)
levels(census_housing$fuel) <- c("Non-solid fuel","Solid fuel")
#check
table(census_housing$fuel_copy, census_housing$fuel)

#water_source
census_housing$water_copy <- census_housing$water
census_housing$water <- as.numeric(census_housing$water)
census_housing$water[census_housing$water %in% c(1,2,3,4,5,8)] <- 0
census_housing$water[census_housing$water %in% c(6,7,9)] <- 1
census_housing$water[census_housing$water %in% c(10)] <- NA
census_housing$water <- as.factor(census_housing$water)
levels(census_housing$water) <- c("Improved water source","Unimproved water source")
#check
table(census_housing$water_copy, census_housing$water)

#toilet_type
census_housing$toilet_copy <- census_housing$toilet
census_housing$toilet <- as.numeric(census_housing$toilet)
census_housing$toilet[census_housing$toilet %in% c(1,3,5,6)] <- 0
census_housing$toilet[census_housing$toilet %in% c(2,4,7,8,9)] <- 1
census_housing$toilet[census_housing$toilet %in% c(10)] <- NA
census_housing$toilet <- as.factor(census_housing$toilet)
levels(census_housing$toilet) <- c("Improved toilet facility","Unimproved toilet facility")
#check
table(census_housing$toilet_copy, census_housing$toilet)

#delete copies
census_housing <- census_housing[, !(names(census_housing) %in% c("structure_copy", "fuel_copy", "water_copy", "toilet_copy"))]

## persons file
census_persons <- read.dta("Data/Census PUMS/Original data/PERSONS RECORDS.dta")
names(census_persons) <- tolower(names(census_persons))
keep_persons <- c("region", "constituency", "urban_rural", "hh_type", "household_serial_number", 
                  "b1", "b3", "b4", "b5", "d3", "pwgt")
census_persons <- census_persons[, (names(census_persons) %in% keep_persons)]
names(census_persons) <- c("region", "constituency", "urban_rural", "hh_type", "household_serial_number", 
                           "line_number", "relationship", "sex", "age", "education", "pwgt")

#recategorize relationship
census_persons$relationship_copy <- census_persons$relationship
census_persons$relationship <- as.numeric(census_persons$relationship)
census_persons$relationship[census_persons$relationship %in% c(3:4)] <- 3
census_persons$relationship[census_persons$relationship %in% c(5)] <- 4
census_persons$relationship[census_persons$relationship %in% c(6,7)] <- 5
census_persons$relationship[census_persons$relationship %in% c(8,9,99)] <- 6
census_persons$relationship <- as.factor(census_persons$relationship)
levels(census_persons$relationship) <- c( "0",
                                         "head",
                                         "spouse",
                                         "child",
                                         "gchild",
                                         "extend",
                                         "other")

#check
table(census_persons$relationship_copy, census_persons$relationship)
census_persons <- census_persons[, !(names(census_persons) %in% c("relationship_copy"))]

#education question not asked to children under 5, we can safely fill in no formal for children in this age group.
census_persons$education[census_persons$age %in% c("0", "1 - 4")] <- "No formal"
#check
summary(census_persons$education[census_persons$age %in% c("0", "1 - 4")])

# merge
census_total <- join(census_housing,census_persons,
                     by=c("region", "constituency", "urban_rural", "hh_type", "household_serial_number"), 
                     type = "full", match = "all") 

#select only conventional households in Oshikoto
census_total <- census_total[(census_total$hh_type == "Conventional households" & census_total$region == "Oshikoto"),]
census_total <- census_total[, !(names(census_total) %in% c("hh_type", "region"))]

#check
table(census_total$hwgt, census_total$pwgt)
census_total <- census_total[, !(names(census_total) %in% c("pwgt"))]
census_total$hwgt <- as.numeric(levels(census_total$hwgt)[census_total$hwgt])
  
## gen unique household and person id
## (if also other types of households and/or other regions are selected, 
## unique hh id will be a combination of region, constituency, hh_type, hh_serial_number)
census_total$hhid <- paste(census_total$constituency, census_total$household_serial_number)
census_total$pid <- paste(census_total$hhid, census_total$line_number)
census_total <- census_total[, !(names(census_total) %in% c("household_serial_number","line_number"))]
census_total <- ddply(census_total,.(hhid),transform,hhsize = length(pid))

## add hhhead variables
census_total$sex_hhhead <- census_total$sex
census_total <- ddply(census_total,.(hhid),transform, sex_hhhead = sex_hhhead[1])
census_total$age_hhhead <- census_total$age
census_total <- ddply(census_total,.(hhid),transform, age_hhhead = age_hhhead[1])

## transform sleeping_rooms to space 
census_total$ind_room <- census_total$hhsize/census_total$sleeping_rooms
census_total$space <- ifelse(census_total$ind_room<=3,0,1)
census_total$space[is.na(census_total$hhsize)] <- NA
census_total$space[is.na(census_total$sleeping_rooms)] <- NA
census_total$space <- as.factor(census_total$space)
levels(census_total$space) <- c( "Adequate space",
                                    "Inadequate space")
census_total <- census_total[, !(names(census_total) %in% c("sleeping_rooms", "ind_room"))]

census_total$hhsize <- factor(census_total$hhsize)
#sanity check, households with fewer than 4 individuals can never have inadequate space
round(prop.table(table(census_total$space, census_total$hhsize)), digits = 3)

census_total$slum <- ifelse((census_total$water == "Unimproved water source" |
                             census_total$toilet == "Unimproved toilet facility" |
                             census_total$structure == "Non-durable floor" |
                             census_total$space == "Inadequate space"),1,0)

#add names of constituencies in Oshikoto
census_total$constituency <- as.factor(census_total$constituency)
levels(census_total$constituency) <- c("Engodi", "Guinas", "Okankolo", "Olukonda", "Omuntele", 
                                       "Omuthiyagwipundi", "Onayena", "Oniipa","Onyaanya","Tsumeb")          

#define the number of households that should be sampled per constituency according the the household point location file.
oshikoto_hh <- readOGR("Data/const_2011_oshikoto_v7/const_2011_oshikoto_v7.shp")
#create 5% extra household so that we are sure that even with rounding we have sufficient households 
# to allocate to household locations by constituency/urban,rural.
oshikoto_hh$SUM_HH_plus <- oshikoto_hh$SUM_HH*1.05
ipu_cons <- as.list(oshikoto_hh$SUM_HH_plus)
names(ipu_cons) <- oshikoto_hh$Constituen

##### 3. generate synthetic data

#Reweight to the number of households per constituency corresponding to household point locations.
census_total_hh <- census_total[census_total$relationship %in% c("head"),]
ipu_inp <- data.table(model.matrix(~constituency +0, data=census_total_hh))
colnames(ipu_inp) <- c(names(ipu_cons))

ipu_weights <- ipu(inp=ipu_inp, con=ipu_cons, verbose=TRUE)
ipu_weights$hhid <- census_total_hh$hhid
ipu_weights <- ipu_weights[,names(ipu_weights) %in% c("hhid", "weights")]
census_total <- ddply(merge(census_total, ipu_weights), .(hhid))

#specify input dataset
#stratify by urban/rural
#(stratification by constituency would also be an option, but was in this case not possible, because 
# some regions have no households with an unimproved water source, so that this varaible cannot be estimated.)
siminput <- specifyInput(census_total, hhid = "hhid", pid = "pid", hhsize = "hhsize", weight = "weights", strata = "urban_rural") 
siminput

#generate household structure
synthP <- simStructure(data=siminput,
                       method="direct",
                       basicHHvars=c("constituency", "urban_rural", "relationship", "hhsize", "sex", "age", "sex_hhhead", "age_hhhead"))
synthP

table(popData(synthP)[popData(synthP)$relationship == "head"]$constituency)
oshikoto_hh$SUM_HH
# total number of households in synthetic population: 39162
# total number of households in location file: 37298
# by constituency mismatch of +190, +22, +246, +170, +148, +338, +81, +313, +154, +202. (so sufficient hh's for each constituency!)

#add household variables: housing_type, tenure, water, toilet
synthP <- simRelation(synthP, relation = "relationship", head = "head", direct = c("spouse", "child (in law)", "grandchild", "parent", "other"),
  additional = c("fuel", "structure", "toilet", "water"))

#add household variables: space with hhsize restrictions
hhsize.1 <- list()
for (i in c(1:length(levels(census_total$hhsize)))) {
  hhsize.1[[i]] <- levels(census_total$space)
}
names(hhsize.1) <- levels(census_total$hhsize)
hhsize.1[[1]] <- hhsize.1[[2]] <- hhsize.1[[3]] <-  levels(census_total$space)[c(2)]
synthP <- simRelation(synthP, relation = "relationship", head = "head", direct = c("spouse", "child (in law)", "grandchild", "parent", "other"),
                      additional = c("space"), 
                      limit = list("hhsize.1"))
#check restriction
table(popData(synthP)$space, popData(synthP)$hhsize)
#no problems
rm(hhsize.1)

#add ind level variables - education with age restrictions
age <- list()
for (i in c(1:length(levels(census_total$age)))) {
  age[[i]] <- levels(census_total$education)
}
names(age) <- levels(census_total$age)
age[[1]] <- age[[2]] <- levels(census_total$education)[c(1)]
age[[3]] <- levels(census_total$education)[c(1,2)]
age[[4]] <- levels(census_total$education)[c(1,2,3)]
age[[5]] <- levels(census_total$education)[c(1,2,3,4)]

synthP <- simRelation(synthP, relation = "relationship", head = "head", additional = c("education"), 
                     limit = list("age"))
#check restriction
table(popData(synthP)$age, popData(synthP)$education)
#A very small number of young people still end op in complete tertiary, which should not be possible
# --> recode them later to highest possible attained education in that age range
rm(age)

synth_pop <- popData(synthP)

#recode young individuals in tertiary
synth_pop$education[(synth_pop$age %in% c("0", "1 - 4") & synth_pop$education %in% c("Complete tertiary"))] <- "No formal"
synth_pop$education[(synth_pop$age %in% c("5 - 9") & synth_pop$education %in% c("Complete tertiary"))] <- "Incomplete primary"
synth_pop$education[(synth_pop$age %in% c("10 - 14") & synth_pop$education %in% c("Complete tertiary"))] <- "Complete primary"
synth_pop$education[(synth_pop$age %in% c("15 - 19") & synth_pop$education %in% c("Complete tertiary"))] <- "Complete secondary"
#check 
table(synth_pop$age, synth_pop$education)

synth_pop$slum <- ifelse((synth_pop$water == "Unimproved water source" |
                          synth_pop$toilet == "Unimproved toilet facility" |
                          synth_pop$structure == "Non-durable floor" |
                          synth_pop$space == "Inadequate space"),1,0)


#### 4. Validation

# check if per variable counts are close to what would be expected from the census
for (var in names(census_total)[!names(census_total) %in% c("hhid", "hwgt", "pid", "weights")]) {
  print(var)
  diff_prop <- round(prop.table(tableWt(eval(parse(text = paste("census_total$", var, sep = ""))), weights = census_total$weights)), digits = 3) - 
    round(prop.table(table(eval(parse(text = paste("synth_pop$", var, sep = ""))))), digits = 3)
  print(diff_prop)
}
# no big differences. Biggest difference in toilet -0.4% improved 

#### 5. Make cluster means

# from ind. to hh head in corresponds with step 1 file. 
# propotion under 5
synth_pop$u5 <- ifelse(synth_pop$age %in% c("0", "1 - 4"), 1, 0)
synth_pop<- ddply(synth_pop,.(hhid), transform, prop_u5 = mean(u5))

#hh_structure 
synth_pop$spouse <- ifelse(synth_pop$relationship=="spouse", 1, 0)
synth_pop$child <- ifelse(synth_pop$relationship %in% c("child"), 1, 0)
synth_pop$gchild <- ifelse(synth_pop$relationship=="gchild", 1, 0)
synth_pop$extend <- ifelse(synth_pop$relationship %in% c("extend"), 1, 0)
synth_pop$other <- ifelse(synth_pop$relationship %in% c("other"), 1, 0)
synth_pop <- ddply(synth_pop,.(hhid), transform, 
                   any_spouse = max(spouse), 
                   any_child = max(child),
                   any_gchild = max(gchild),
                   any_extend = max(extend), 
                   any_other = max(other), 
                   any_u5 = max(u5))

synth_pop$educ_hhhead <- ifelse(synth_pop$relationship %in% c("head"), synth_pop$education, NA)
synth_pop <- ddply(synth_pop,.(hhid), transform, 
                   educ_hhhead = getmode(educ_hhhead))
synth_pop$educ_hhhead <- as.factor(synth_pop$educ_hhhead)
levels(synth_pop$educ_hhhead) <- levels(synth_pop$education)

#select only hhead
synth_pop_hh <- synth_pop[synth_pop$relationship == "head",]

synth_pop_hh$noeduc <- ifelse(synth_pop_hh$educ_hhhead == "No formal", 1, 0) 
synth_pop_hh$higheduc <- ifelse(synth_pop_hh$educ_hhhead %in% c("Complete secondary", "Complete tertiary"), 1, 0) 
synth_pop_hh$single <- ifelse(synth_pop_hh$hhsize == 1, 1, 0) 

dim(synth_pop_hh) # n= 37299 36

# make cluster types (using DHS kmeans)
head(synth_pop_hh)
synth_pop_hh$urban_rural <- as.numeric(synth_pop_hh$urban_rural)-1
# 1 is rural, 0 is urban
synth_pop_hh$structure <- as.numeric(synth_pop_hh$structure) - 1
synth_pop_hh$fuel <- as.numeric(synth_pop_hh$fuel) - 1
synth_pop_hh$water <- as.numeric(synth_pop_hh$water) - 1
synth_pop_hh$toilet <- as.numeric(synth_pop_hh$toilet) - 1
synth_pop_hh$space <- as.numeric(synth_pop_hh$space) - 1

#drop individual level variables 
synth_pop_hh <- synth_pop_hh[, !names(synth_pop_hh) %in% c("pid", "relationship", "age", "sex", "education", "spouse", "child", "gchild", "extend", "other", "u5")]
dim(synth_pop_hh) # n= 37299 25

# Note, we don't need to center and rescale the variables. 

## load DHS kmeans
library(FNN)

cluster_var <- c("urban_rural", "noeduc", "any_u5", "toilet", "water", "structure", "space", "fuel")
cutline <- 0.65
km <- readRDS("Output/nam_dhs_kmeans.rds")
head(km)

# estimate kmeans cluster based on distance to centers
pred.clust <- get.knnx(km$centers, synth_pop_hh[,cluster_var], 1)$nn.index[,1]
table(pred.clust)
prop.table(table(pred.clust))

# load reclassified
k.hc <- readRDS("Output/nam_dhs_cutkmeans.rds")
plot(k.hc, hang=-1)

# cut/regroup the kmeans into 7 classes based on dendrogram
k.hc.cut <- cutree(k.hc, h=cutline) # cross-walk table
k.hc.cut <- unname(k.hc.cut) # drop labels

# apply to data with reclassified clusters
synth_pop_hh$hhclass <- k.hc.cut[pred.clust]
head(synth_pop_hh)

#check if urban/rural classification makes sense with respect to constituency and urban/rural variable
table(synth_pop_hh$constituency, synth_pop_hh$hhclass)
table(synth_pop_hh$urban_rural, synth_pop_hh$hhclass)

gc()

## output population data - each population stored in seperate folder "synth_pop_x" ##
# household-level records
# use households for spatial allocation
saveRDS(synth_pop_hh, file="Output/populations/synth_pop_1/nam_synth_hh.rds")
# person-level records
saveRDS(synth_pop, file="Output/populations/synth_pop_1/nam_synth_person.rds")

save.image(file = "NAM_stepB5.RData")
